@@ -6,7 +6,7 @@ from contextlib import ExitStack
 from pathlib import Path
 from typing import BinaryIO, Dict, Optional, Generator, Iterator, Set
 
-from mercury_engine_data_structures import crc, formats, dread_data
+from mercury_engine_data_structures import formats, dread_data
 from mercury_engine_data_structures.formats.base_resource import AssetId, BaseResource, NameOrAssetId, resolve_asset_id
 from mercury_engine_data_structures.formats.pkg import PKGHeader, Pkg
 from mercury_engine_data_structures.game_check import Game
@@ -15,34 +15,35 @@ from mercury_engine_data_structures.game_check import Game
 class PkgEditor:
     """
     Manages efficiently reading all PKGs in the game and writing out modifications to a new path.
+
+    _files_for_asset_id: mapping of asset id to all pkgs it can be found at
+    _ensured_asset_ids: mapping of pkg name to assets we'll copy into it when saving
+    _modified_resources: mapping of asset id to bytes. When saving, these asset ids are replaced
     """
     _files_for_asset_id: Dict[AssetId, Set[str]]
+    _ensured_asset_ids: Dict[str, Set[AssetId]]
     _modified_resources: Dict[AssetId, bytes]
 
-    def __init__(self, files: Dict[str, BinaryIO], target_game: Game = Game.DREAD):
-        self.files = files
-        self.target_game = target_game
-        self.headers = {
-            name: PKGHeader.parse_stream(file, target_game=target_game)
-            for name, file in files.items()
-        }
-        self._files_for_asset_id = collections.defaultdict(set)
-        for name, header in self.headers.items():
-            for entry in header.file_entries:
-                self._files_for_asset_id[entry.asset_id].add(name)
-        self._modified_resources = {}
-
-    @classmethod
-    @contextlib.contextmanager
-    def open_pkgs_at(cls, root: Path) -> Generator["PkgEditor", None, None]:
+    def __init__(self, root: Path, target_game: Game = Game.DREAD):
         all_pkgs = root.rglob("*.pkg")
 
-        with ExitStack() as stack:
-            files = {
-                file.relative_to(root).as_posix(): stack.enter_context(file.open("rb"))
-                for file in all_pkgs
-            }
-            yield PkgEditor(files)
+        self.files = {}
+        self.root = root
+        self.target_game = target_game
+        self.headers = {}
+        self._files_for_asset_id = collections.defaultdict(set)
+        self._ensured_asset_ids = {}
+        self._modified_resources = {}
+
+        for pkg_path in all_pkgs:
+            name = pkg_path.relative_to(root).as_posix()
+            self.files[name] = pkg_path
+            with pkg_path.open("rb") as f:
+                self.headers[name] = PKGHeader.parse_stream(f, target_game=target_game)
+
+            self._ensured_asset_ids[name] = set()
+            for entry in self.headers[name].file_entries:
+                self._files_for_asset_id[entry.asset_id].add(name)
 
     def all_asset_ids(self) -> Iterator[AssetId]:
         """
@@ -74,14 +75,14 @@ class PkgEditor:
 
             for entry in header.file_entries:
                 if entry.asset_id == asset_id:
-                    self.files[name].seek(entry.start_offset)
-                    return self.files[name].read(entry.end_offset - entry.start_offset)
+                    with self.files[name].open("rb") as f:
+                        f.seek(entry.start_offset)
+                        return f.read(entry.end_offset - entry.start_offset)
 
         raise ValueError(f"Unknown asset_id: {asset_id:0x}")
 
     def get_parsed_asset(self, name: str, in_pkg: Optional[str] = None) -> BaseResource:
-        asset_id = crc.crc64(name)
-        data = self.get_raw_asset(asset_id, in_pkg)
+        data = self.get_raw_asset(name, in_pkg)
         file_format = os.path.splitext(name)[1][1:]
         return formats.format_for(file_format).parse(data, target_game=self.target_game)
 
@@ -90,20 +91,46 @@ class PkgEditor:
             new_data = new_data.build()
         self._modified_resources[resolve_asset_id(asset_id)] = new_data
 
-    def save_modified_pkgs(self, out: Path):
+    def ensure_present(self, pkg_name: str, asset_id: NameOrAssetId):
+        """
+        Ensures the given pkg has the give assets, collecting from other pkgs if needed.
+        """
+        if pkg_name not in self._ensured_asset_ids:
+            raise ValueError(f"Unknown pkg_name: {pkg_name}")
+        asset_id = resolve_asset_id(asset_id)
+
+        # If the pkg already has the given asset, do nothing
+        if pkg_name not in self._files_for_asset_id[asset_id]:
+            self._ensured_asset_ids[pkg_name].add(asset_id)
+
+    def save_modified_pkgs(self):
         modified_pkgs = set()
         for asset_id in self._modified_resources.keys():
             modified_pkgs.update(self._files_for_asset_id[asset_id])
 
+        # Read all asset ids we need to copy somewhere else
+        asset_ids_to_copy = {}
+        for asset_ids in self._ensured_asset_ids.values():
+            for asset_id in asset_ids:
+                if asset_id not in asset_ids_to_copy:
+                    asset_ids_to_copy[asset_id] = self.get_raw_asset(asset_id)
+
         for pkg_name in modified_pkgs:
-            self.files[pkg_name].seek(0)
-            pkg = Pkg.parse_stream(self.files[pkg_name], target_game=self.target_game)
+            with self.files[pkg_name].open("rb") as f:
+                pkg = Pkg.parse_stream(f, target_game=self.target_game)
 
             for asset_id, data in self._modified_resources.items():
                 if pkg_name in self._files_for_asset_id[asset_id]:
                     pkg.replace_asset(asset_id, data)
 
-            pkg_out = out.joinpath(pkg_name)
-            pkg_out.parent.mkdir(parents=True, exist_ok=True)
-            with pkg_out.open("wb") as f:
+            for asset_id in self._ensured_asset_ids[pkg_name]:
+                pkg.add_asset(asset_id, asset_ids_to_copy[asset_id])
+                self._files_for_asset_id[asset_id].add(pkg_name)
+
+            with self.files[pkg_name].open("wb") as f:
                 pkg.build_stream(f)
+
+            # Clear the ensured asset ids, since we've written these
+            self._ensured_asset_ids[pkg_name] = set()
+
+        self._modified_resources = {}
